package com.streambase.contrib.adapters.kxsystems;

import java.beans.*;

import com.streambase.sb.operator.Operator;
import com.streambase.sb.operator.parameter.*;

/**
 * A BeanInfo class controls what properties are exposed, add 
 * metadata about properties (such as which properties are optional), and access 
 * special types of properties that can't be automatically derived via reflection. 
 * If a BeanInfo class is present, only the properties explicitly declared in
 * this class will be exposed by StreamBase.
 */
public class QdbcReaderBeanInfo extends SimpleBeanInfo {

	/*
	 * The order of properties below determines the order they are displayed within
	 * the StreamBase Studio property view. 
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		try {
			PropertyDescriptor[] p = {
					new SBPropertyDescriptor("hostname", QdbcReader.class)
						.displayName("Hostname").description(""),
					new SBPropertyDescriptor("port", QdbcReader.class)
						.displayName("Port").description(""),
					new SBPropertyDescriptor("username", QdbcReader.class)
						.displayName("User Name").description("").optional(),
					new SBPropertyDescriptor("password", QdbcReader.class)
						.displayName("Password").description("").optional(),
					new SBPropertyDescriptor("useTLS", QdbcReader.class)
						.displayName("UseTLS").description("").optional(),
					new SBPropertyDescriptor("metaQuery", QdbcReader.class)
						.displayName("MetaQuery").description(""),
					new SBPropertyDescriptor("queryTable", QdbcReader.class)
						.displayName("Query Table").description(""),
					new EnumPropertyDescriptor("Mode", QdbcReader.class,
						new String[] { QdbcReader._ENUM_PROP_BLOCKING_,
									   QdbcReader._ENUM_PROP_STREAMING_ })
						.displayName("Mode").description(""),
					new SBPropertyDescriptor("pollingMillis", QdbcReader.class)
						.displayName("Polling period")
						.description("Frequency in milliseconds to issue MetaQuery (0 = once only (default))")
						.optional(),
					
		            new JavaEnumPropertyDescriptor<Operator.LogLevel>("LogLevel", QdbcReader.class, Operator.LogLevel.class).optional()
	                	.displayName("Log Level")
	                	.description("The log level used by the adapter. Operates independently of the top-level StreamBase log level value."),
					
					new SBPropertyDescriptor("schema0", QdbcReader.class)
					};

			return p;
		} catch (IntrospectionException e) {
			System.err.println("Failed to create property descriptors");
			e.printStackTrace(); // @NOTE: Use logger. This is totally useless in a production environment.
			return null;
		}
	}

}
