package com.streambase.contrib.adapters.kxsystems;

import java.io.IOException;

import kx.c;
import kx.c.KException;

import com.streambase.sb.*;
import com.streambase.sb.Schema.Field;
import com.streambase.sb.operator.*;

import static com.streambase.contrib.adapters.kxsystems.QdbcConstants.*;

/**
 * Boilerplate generated by JDT StreamBase Client Templates (Version: 5.2.0.804152100).
 *
 * Output adapters take streams from an application and make them
 * available outside of the application. Any implementation of 
 * this class <b>must</b> extend class OutputAdapter. Output adapters
 * should override method processTuple(int, Tuple) to receive tuples 
 * from the application.
 * <p>
 * For in-depth information on implementing a custom adapter, please
 * see "Developing StreamBase Embedded Adapters" in the StreamBase documentation.
 * <p>
 * 
 * '@Note' meta-annotation comments are employed where noteworthy choices have been
 * taken and explanations are worth sharing.
 * 
 * @author Darach Ennis (darach.ennis@streambase.com)
 *
 */
public class QdbcWriter extends QdbcWriterBase {

	public static final long serialVersionUID = 1208823435774L;

	// Properties
	private String Operation;

	// Constants for enum property conversionType
	public static String _ENUM_PROP_INSERT_ = "INSERT";
	public static String _ENUM_PROP_UPSERT_ = "UPSERT";
	public static String _ENUM_PROP_DELETE_ = "DELETE";

	private String table;
	private String displayName = "Qdbc Writer";

	// State
	private c.Flip qdbcMeta = null;

	/**
	 * The constructor is called when the Adapter instance is created, but before the Adapter 
	 * is connected to the StreamBase application. We recommended that you set the initial input 
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount). 
	 * The default is no input ports or output ports. The constructor may also set default values for 
	 * adapter parameters. These values will be displayed in StreamBase Studio when a new adapter is 
	 * dragged to the canvas, and serve as the default values for omitted optional parameters.
	 */
	public QdbcWriter() {
		super();
		logger = getLogger();
		setPortHints(1, 0);
		setDisplayName(displayName);
		setHostname("localhost");
		setPort(DEFAULT_QDBC_PORT);
		setTable("");
		setLogLevel(LogLevel.INFO);
	}

	/**
	 * Typecheck this adapter. The typecheck method is called after the adapter instance is
	 * connected in the StreamBase application. The adapter should validate its parameters 
	 * and throw TypecheckException if any problems are found. The message associated with 
	 * the thrown TypecheckException will be displayed in StreamBase Studio during authoring,
	 * or printed on the console by sbd. Input adapters should set the schema of each output
	 * port by calling the setOutputSchema(portNum, schema) method for each output port. 
	 * If the adapter needs to change the number of input ports based on parameter values,
	 * it should call requireInputPortCount(portCount) at this point. 
	 */
	public void typecheck() throws TypecheckException {
		if (table == null || table.trim().equalsIgnoreCase("")) {
			throw new TypecheckException("Qdbc table must be a valid Q table");
		}
	
		conn = checkAndConnect();

		if (qdbcMeta == null) {
			final StringBuilder sb = new StringBuilder();
			sb.append("select from ");
			sb.append(getTable());
			sb.append(" where 1 = 0");
			final String testTableExists = sb.toString();
			try {
				qdbcMeta = (c.Flip)conn.k(testTableExists);
			} catch (KException e) {
				logger.error("Table " + getTable() + " does not exist or is not a supported endpoint", e);
				throw new TypecheckException("Qdbc query must be a valid Q query", e);
			} catch (IOException e) {
				logger.error("Unable to connect to Q host at " + getHostname() + ":" + getPort(), e);
				throw new TypecheckException("Qdbc query must be a valid Q query", e);
			} 
			
			try {
				schema0 = QdbcHelper.deduceSchema(conn, "_"+getName()+":"+getTable()+"Schema", qdbcMeta, getInputSchema(0));
			} catch (KException e) {
				logger.error("Unable to deduce schema from Qdbc metadata query", e);
				throw new TypecheckException("Qdbc query must use StreamBase compatible types.", e);
			} catch (IOException e) {
				logger.error("Unable to deduce schema from Qdbc metadata query", e);
				throw new TypecheckException("Qdbc query must use StreamBase compatible types.", e);
			} catch (TupleException e) {
				logger.error("Unable to deduce schema from Qdbc metadata query", e);
				throw new TypecheckException("Qdbc query must use StreamBase compatible types.", e);
			}
			
			Schema inputSchema = getInputSchema(0);
			validate(schema0, inputSchema);
		}
	}

	private void validate(Schema schema0, Schema inputSchema) throws TypecheckException {
		if (schema0.getFieldCount() != inputSchema.getFieldCount()) {
			throw new TypecheckException("Schemata MUST be equal.");
		}
		for (int i = 0; i < schema0.getFieldCount(); i++) {
			if (!schema0.getField(i).getDataType().equals(inputSchema.getField(i).getDataType())) {
				throw new TypecheckException("Schemata MUST be equal.");
			}
		}
	}

	/**
	 * This method will be called by the StreamBase server for each
	 * Tuple given to the adapter to process.  Output adapters should
	 * override this method to process tuples.
	 *<p>
	 * The default implementation does nothing.
	 * 
	 * @param inputPort the input port that the tuple is from (ports are zero based)
	 * @param tuple the tuple from the given input port
	 * @throws StreamBaseException Terminates the application. 
	 * @throws IOException 
	 * @throws KException 
	 */
	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {

		if (logger.isDebugEnabled())
			logger.debug(getName()+ ": processTuple " + tuple);

		if (_ENUM_PROP_UPSERT_.equals(getOperation())) {
			try {
				upsert(inputPort, tuple);
			} catch (KException e) {
				logger.error("Unable to perform Qdbc upsert", e);
				throw new StreamBaseException(e);
			} catch (IOException e) {
				logger.error("Unable to perform Qdbc upsert", e);
				throw new StreamBaseException(e);
			}
			return;
		}
		
		if (_ENUM_PROP_INSERT_.equals(getOperation())) {
			try {
				insert(inputPort, tuple);
			} catch (KException e) {
				logger.error("Unable to perform Qdbc insert", e);
				throw new StreamBaseException(e);
			} catch (IOException e) {
				logger.error("Unable to perform Qdbc insert", e);
				throw new StreamBaseException(e);
			}
			return;
		}

		if (_ENUM_PROP_DELETE_.equals(getOperation())) {
			try {
				delete(inputPort, tuple);
			} catch (KException e) {
				logger.error("Unable to perform Qdbc upsert", e);
				throw new StreamBaseException(e);
			} catch (IOException e) {
				logger.error("Unable to perform Qdbc upsert", e);
				throw new StreamBaseException(e);
			}
			return;
		}
}

	private void upsert(int inputPort, Tuple tuple) throws KException, IOException, TypecheckException, TupleException {
		final StringBuilder sb = new StringBuilder();
		sb.append("`");
		sb.append(getTable());
		sb.append(" upsert (");
		sb.append(QdbcHelper.toDelimitedString(qdbcMeta, tuple,';'));
		sb.append(")");
		if (logger.isDebugEnabled()) {
			logger.debug("UPSERT " + sb.toString());
		}
		conn.ks(sb.toString());
		conn.k("");
	}

	private void insert(int inputPort, Tuple tuple) throws IOException, KException, TypecheckException, TupleException {
		final StringBuilder sb = new StringBuilder();
		sb.append("`");
		sb.append(getTable());
		sb.append(" insert (");
		sb.append(QdbcHelper.toDelimitedString(qdbcMeta, tuple,';'));
		sb.append(")");
		if (logger.isDebugEnabled()) {
			logger.debug("INSERT " + sb.toString());
		}
		conn.ks(sb.toString());
		conn.k("");
	}

	private void delete(int inputPort, Tuple tuple) throws TupleException, TypecheckException, KException, IOException {
		final StringBuilder sb = new StringBuilder();
		sb.append("delete from ");
		sb.append(getTable());
		sb.append(" where ");
		for (int i = 0; i < tuple.getTupleLength(); i++) {
			final Field field = (Field)tuple.getField(i);
			sb.append(field.getName());
			sb.append(" = ");
			sb.append(QdbcHelper.getFieldValue(qdbcMeta.y[i].getClass().getComponentType(), tuple, field));
		}
		sb.append(QdbcHelper.toDelimitedString(qdbcMeta, tuple," = ", "; "));
		sb.append(")");
		if (logger.isDebugEnabled()) {
			logger.debug("DELETE " + sb.toString());
		}
		conn.ks(sb.toString());
		conn.k("");
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public void setOperation(String Operation) {
		this.Operation = Operation;
	}

	public String getOperation() {
		return this.Operation;
	}

	public void setTable(String table) {
		this.table = table;
	}

	public String getTable() {
		return this.table;
	}

}
