package com.streambase.contrib.adapters.kxsystems;

import java.beans.*;

import com.streambase.sb.operator.Operator;
import com.streambase.sb.operator.parameter.*;

/**
 * A BeanInfo class controls what properties are exposed, add 
 * metadata about properties (such as which properties are optional), and access 
 * special types of properties that can't be automatically derived via reflection. 
 * If a BeanInfo class is present, only the properties explicitly declared in
 * this class will be exposed by StreamBase.
 */
public class QdbcWriterBeanInfo extends SimpleBeanInfo {

	/*
	 * The order of properties below determines the order they are displayed within
	 * the StreamBase Studio property view. 
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		try {
			PropertyDescriptor[] p = {
					new SBPropertyDescriptor("hostname", QdbcWriter.class)
						.displayName("Hostname").description("").optional(),
					new SBPropertyDescriptor("port", QdbcWriter.class)
						.displayName("Port").description(""),
					new SBPropertyDescriptor("username", QdbcWriter.class)
						.displayName("User Name").description("").optional(),
					new SBPropertyDescriptor("password", QdbcWriter.class)
						.displayName("Password").description("").optional(),
					new SBPropertyDescriptor("useTLS", QdbcWriter.class)
						.displayName("UseTLS").description("").optional(),
					new EnumPropertyDescriptor("Operation", QdbcWriter.class,
							new String[] { QdbcWriter._ENUM_PROP_INSERT_,
									QdbcWriter._ENUM_PROP_UPSERT_,
									QdbcWriter._ENUM_PROP_DELETE_ })
						.displayName("Operation").description(""),
					new SBPropertyDescriptor("table", QdbcWriter.class)
						.displayName("Table").description(""),
					new JavaEnumPropertyDescriptor<Operator.LogLevel>("LogLevel", QdbcWriter.class, Operator.LogLevel.class).optional()
						.displayName("Log Level")
						.description("The log level used by the adapter. Operates independently of the top-level StreamBase log level value."),
			};

			return p;
		} catch (IntrospectionException e) {
			System.err.println("Failed to create property descriptors");
			e.printStackTrace();
			return null;
		}
	}

}
